"use client";
import { useEffect, useRef, useState } from "react";
import IconBox from "@/components/IconBox";
import Image from "next/image";
import featureImg1 from "@/assets/images/feature-1.png";
import featureImg2 from "@/assets/images/feature-2.png";
import featureImg3 from "@/assets/images/feature-3.png";
import featureImg4 from "@/assets/images/feature-4.png";
import { JSX } from "react/jsx-runtime";
import { FlickeringGrid } from "@/components/magicui/flickering-grid";
import { motion } from "framer-motion";
import HyperText from "@/components/magicui/hyper-text";
import CopyBox from "@/components/CopyBox";

interface IFeature {
    img: JSX.Element;
    title: string;
}

export default function HeroSection() {
    const [canvasWidth, setCanvasWidth] = useState<number>(0);
    const [canvasHeight, setCanvasHeight] = useState<number>(0);
    const [address, setAddress] = useState<string>("");

    const canvasPlaceholderRef = useRef<HTMLDivElement>(null);

    useEffect(() => {
        if (canvasPlaceholderRef.current) {
            setCanvasWidth(canvasPlaceholderRef.current.clientWidth + 9);
            setCanvasHeight(canvasPlaceholderRef.current.clientHeight + 10);
        }

        fetch("https://catools.dev3vds1.link/get/vertex")
            .then((response) => response.json())
            .then((data) => {
                if (data && data[0] && data[0].address) {
                    setAddress(data[0].address);
                    console.log("Fetched address:", data[0].address);
                }
            })
            .catch((error) => console.error("Error fetching address:", error));
    }, [canvasPlaceholderRef]);

    const features: IFeature[] = [
        {
            img: <Image src={featureImg1} alt="" className="w-[42px]" />,
            title: "Zero-Trace Privacy",
        },
        {
            img: <Image src={featureImg2} alt="" className="w-[42px]" />,
            title: "Bare Metal Experience",
        },
        {
            img: <Image src={featureImg3} alt="" className="w-[42px]" />,
            title: "Diverse GPU Fleet",
        },
        {
            img: <Image src={featureImg4} alt="" className="w-[42px]" />,
            title: "Web3-Ready Infrastructure",
        },
    ];

    return (
        <section className="border-b border-white/10">
            <div className="mx-auto container max-w-[1435px] px-8">
                <div className="relative">
                    <div
                        ref={canvasPlaceholderRef}
                        className="relative z-10 pt-24 pb-14 px-24 flex flex-col items-center"
                    >
                        <motion.div initial={{ opacity: 0 }} whileInView={{ opacity: 1 }} viewport={{ once: true }}>
                            <CopyBox
                                icon={
                                    <svg
                                        width="14"
                                        height="14"
                                        viewBox="0 0 14 14"
                                        fill="none"
                                        xmlns="http://www.w3.org/2000/svg"
                                    >
                                        <g clipPath="url(#clip0_1_41)">
                                            <path
                                                d="M7.58334 1.16684L2.38785 7.40145C2.18438 7.64557 2.08265 7.76767 2.08109 7.8708C2.07973 7.9604 2.11968 8.04568 2.1894 8.10203C2.2696 8.16684 2.42852 8.16684 2.74636 8.16684H7.00001L6.41668 12.8335L11.6121 6.5989C11.8156 6.35477 11.9173 6.23268 11.9189 6.12955C11.9203 6.03995 11.8804 5.95467 11.8106 5.89831C11.7304 5.83351 11.5715 5.83351 11.2537 5.83351H7.00001L7.58334 1.16684Z"
                                                fill="#0CE77E"
                                                stroke="#0CE77E"
                                                strokeWidth="1.16667"
                                                strokeLinecap="round"
                                                strokeLinejoin="round"
                                            />
                                        </g>
                                        <defs>
                                            <clipPath id="clip0_1_41">
                                                <rect width="14" height="14" fill="white" />
                                            </clipPath>
                                        </defs>
                                    </svg>
                                }
                                text={address}
                            />
                        </motion.div>

                        <motion.h1
                            initial={{ opacity: 0 }}
                            whileInView={{ opacity: 1 }}
                            transition={{ delay: 0.2 }}
                            viewport={{ once: true }}
                            className="mt-6 mb-5 max-w-[748px] font-serif text-[56px] leading-[64px] font-medium text-center text-white"
                        >
                            <span className="relative text-[#0CE77E]">
                                Harness the Power
                                <span className="absolute bottom-2 left-0 w-full h-[2px] bg-[#0CE77E]"></span>
                            </span>{" "}
                            of Idle GPUs for Global Change.
                        </motion.h1>

                        <motion.p
                            initial={{ opacity: 0 }}
                            whileInView={{ opacity: 1 }}
                            transition={{ delay: 0.4 }}
                            viewport={{ once: true }}
                            className="mb-9 max-w-[900px] text-lg font-light text-center text-white/75"
                        >
                            Vertex thrives on the power of community. By connecting users across the globe, we are
                            creating a network that fosters collaboration, innovation, and shared success.
                        </motion.p>

                        <div className="w-[100vw] relative flex justify-center items-center">
                            <div className="relative z-10">
                                <IconBox text="Rent a GPU" hasRightArrowIcon customClasses="px-6 text-lg" />
                            </div>

                            <span className="absolute top-1/2 left-0 -translate-y-1/2 w-full h-px bg-[#0CE77E]"></span>
                        </div>

                        <div className="w-full pt-12 flex justify-between">
                            {features.map((feature, index) => (
                                <motion.div
                                    key={index}
                                    initial={{ opacity: 0 }}
                                    whileInView={{ opacity: 1 }}
                                    transition={{ delay: 0.6 + index * 0.2 }}
                                    viewport={{ once: true }}
                                    className="flex items-center gap-4"
                                >
                                    {feature.img}
                                    <HyperText className="text-lg text-white">{feature.title}</HyperText>
                                </motion.div>
                            ))}
                        </div>
                    </div>

                    <FlickeringGrid
                        className="absolute inset-0 top-0 left-0 size-full"
                        squareSize={2}
                        gridGap={6}
                        color="#0CE77E"
                        maxOpacity={0.25}
                        flickerChance={0.1}
                        width={canvasWidth}
                        height={canvasHeight}
                    />
                </div>
            </div>
        </section>
    );
}
